/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#include "global.h"
#include "game.h"


/* return a valid slot for a connection     *
 *   else return FALSE                      */
gboolean connect_get_valid_id(guint16 *idconnect)  {

	guint16 i;

	for(i = 1; i < MAX_CONNECTION; i++)
		if(!connection_is_open[i]) {

			*idconnect = i;
			return(TRUE);
		}

	return(FALSE);
}


/* insert test in Message Box */ 
void text_insert_message(gchar* text, guint32 lenght)  {

	guint32 i, len;

	gtk_text_freeze( GTK_TEXT(global->Message) );

	len = gtk_text_get_length( GTK_TEXT(global->Message) );
	gtk_text_set_point( GTK_TEXT(global->Message), len);
	if(len > 0)  gtk_text_insert(GTK_TEXT(global->Message), NULL, NULL, NULL, "\n", 1);
	gtk_text_insert(GTK_TEXT(global->Message), NULL, NULL, NULL, text, lenght);
	global->message_nb_lines++;

	while(global->message_nb_lines > config->message_max_lines) {

		len = gtk_text_get_length( GTK_TEXT(global->Message) );

		for(i = 0 ;  GTK_TEXT_INDEX( GTK_TEXT(global->Message), i) != '\n' ; i++)  {

			if(i >= len)  { /* argl... no CR founded */

				gtk_text_set_point( GTK_TEXT(global->Message), 0);
				gtk_text_forward_delete( GTK_TEXT(global->Message), len);
				global->message_nb_lines = 0;
				break;
			}
		}

		if(global->message_nb_lines)  {

			i++;
			gtk_text_set_point( GTK_TEXT(global->Message), 0);
			gtk_text_forward_delete( GTK_TEXT(global->Message), i);
			global->message_nb_lines--;
		}
	}

	len = gtk_text_get_length( GTK_TEXT(global->Message) );
	gtk_text_set_point( GTK_TEXT(global->Message), len);

	gtk_text_thaw( GTK_TEXT(global->Message) );

	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( global->Message )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( global->Message )->vadj )->upper );
}


/* insert test in Chat Box */ 
void text_insert_chat(gchar* text, guint32 lenght)  {

	guint32 i, len;

	gtk_text_freeze( GTK_TEXT(game->Chat) );

	len = gtk_text_get_length( GTK_TEXT(game->Chat) );
	gtk_text_set_point( GTK_TEXT(game->Chat), len);
	if(len > 0)  gtk_text_insert(GTK_TEXT(game->Chat), NULL, NULL, NULL, "\n", 1);
	gtk_text_insert(GTK_TEXT(game->Chat), NULL, NULL, NULL, text, lenght);
	game->chat_nb_lines++;

	while(game->chat_nb_lines > config->chat_max_lines) {

		len = gtk_text_get_length( GTK_TEXT(game->Chat) );

		for(i = 0 ;  GTK_TEXT_INDEX( GTK_TEXT(game->Chat), i) != '\n' ; i++)  {

			if(i >= len)  { /* argl... no CR founded */

				gtk_text_set_point( GTK_TEXT(game->Chat), 0);
				gtk_text_forward_delete( GTK_TEXT(game->Chat), len);
				game->chat_nb_lines = 0;
				break;
			}
		}

		if(game->chat_nb_lines)  {

			i++;
			gtk_text_set_point( GTK_TEXT(game->Chat), 0);
			gtk_text_forward_delete( GTK_TEXT(game->Chat), i);
			game->chat_nb_lines--;
		}
	}

	len = gtk_text_get_length( GTK_TEXT(game->Chat) );
	gtk_text_set_point( GTK_TEXT(game->Chat), len);

	gtk_text_thaw( GTK_TEXT(game->Chat) );

	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj )->upper );
}


/* alloc all memory for a game */
void game_alloc()  {

	game = g_malloc0(sizeof (_game) );
}


/* free all memory for a game */
void game_free()  {

	g_free( game );
}


/* return a valid game id */
gboolean game_get_valid_player_slot(guint8 *playerslot)  {

	guint8 i;

	for(i = 0; i < MAX_PLAYERS; i++)
		if(!game->player[i].playerid) {

			*playerslot = i;
			return TRUE;
		}

	return FALSE;
}


/* return the player slot of playerid */
gint8 get_player_slot_with_playerid(guint32 playerid)  {

	guint32 i;

	for(i = 0 ; i < MAX_PLAYERS ; i++)  {

		if(game->player[i].playerid == playerid) {

			return(i);
			break;
		}
	}

	return(-1);
}


gint8 get_playerlistcard_id_with_estate(guint32 estate)  {

	guint32 i;

	for(i = 0 ; i < data->number_playerlist_card ; i++)  {

		if(data->playerlist_card[i].estateid == estate) {

			return(i);
			break;
		}
	}

	return(-1);
}


/* return a valid command slotid */
gboolean game_get_valid_command_slot(guint8 *commandslot)  {

	guint8 i;

	for(i = 0; i < MAX_COMMANDS ; i++)
		if(!game->command[i].open) {

			*commandslot = i;
			return TRUE;
		}

	return FALSE;
}


/* return commandslot if button command is displayed 
   return <0 if button not is not displayed           */
gint8 get_command_button_slot_with_command(guchar *command) {

	guint32 i;

	for(i = 0 ; i < MAX_COMMANDS ; i ++)  {

		if(game->command[i].open)  {

			if(! strncmp(command, game->command[i].command, strlen(command)) )
				return i;
		}
	}

	return -1;
}
