/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <gtk/gtk.h>

#include <libxml/parser.h>
#include <png.h>

#include "config.h"

#include "load.h"
#include "global.h"
#include "readpng.h"



/* parse the data file config.xml */
gboolean xmlparse_file_config()  {

	xmlDocPtr doc;
	xmlNodePtr cur;
	guchar *filename;

	filename = g_strconcat(PACKAGE_DATA_DIR, "/config.xml", NULL);
	doc = xmlParseFile(filename);
	g_free(filename);

	if(doc == NULL)  return FALSE;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		xmlFreeDoc(doc);
		return FALSE;
	}

	if( xmlStrcmp(cur->name, DATAFILE_XMLROOTELEMENT) )  {
		xmlFreeDoc(doc);
		return FALSE;
	}

	cur = cur->xmlChildrenNode; 
	do {
		if(! xmlStrcmp(cur->name, "nick") )  {

			config->nickname = g_strdup( xmlGetProp(cur, "name") );
		}

		if(! xmlStrcmp(cur->name, "metaserver") )  {

			config->metaserver_host = g_strdup( xmlGetProp(cur, "host") );
			config->metaserver_port = atoi( xmlGetProp(cur, "port") );
			config->metaserver_autogetlist = atoi( xmlGetProp(cur, "autogetlist") );
			config->metaserver_autogetgames = atoi( xmlGetProp(cur, "autogetgames") );
		}

		if(! xmlStrcmp(cur->name, "getgames") )  {

			config->getgames_host = g_strdup( xmlGetProp(cur, "host") );
			config->getgames_port = atoi( xmlGetProp(cur, "port") );
			config->getgames_autogetgames = atoi( xmlGetProp(cur, "autogetgames") );
		}

		if(! xmlStrcmp(cur->name, "message") )  {

			config->message_max_lines = atoi( xmlGetProp(cur, "max_lines") );
		}

		if(! xmlStrcmp(cur->name, "chat") )  {

			config->chat_max_lines = atoi( xmlGetProp(cur, "max_lines") );
		}

		if(! xmlStrcmp(cur->name, "game") )  {

			if(! xmlStrcmp( xmlGetProp(cur, "playerlist_position"), "left") )
				config->game_playerlist_position = GAME_PLAYERLIST_POS_LEFT;
			if(! xmlStrcmp( xmlGetProp(cur, "playerlist_position"), "right") )
				config->game_playerlist_position = GAME_PLAYERLIST_POS_RIGHT;
		}


		cur = cur -> next;

	} while(cur != NULL);

	xmlFreeDoc(doc);
	return TRUE;
}


/* parse the data file interface.xml */
gboolean xmlparse_file_interface()  {

	xmlDocPtr doc;
	xmlNodePtr cur, cur2;
	guint8 ident = 0xff;
	guint32 color;
	gint32 xoffset, yoffset;
	guchar *filename;

	filename = g_strconcat(PACKAGE_DATA_DIR, "/interface.xml", NULL);
	doc = xmlParseFile(filename);
	g_free(filename);

	if(doc == NULL)  return FALSE;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		xmlFreeDoc(doc);
		return FALSE;
	}

	if( xmlStrcmp(cur->name, DATAFILE_XMLROOTELEMENT) )  {
		xmlFreeDoc(doc);
		return FALSE;
	}

	cur = cur->xmlChildrenNode; 
	do {

		/* ===== PNGFILES ===== */
		/* ==================== */

		/* parse pngfile_logo */
		if(! xmlStrcmp(cur->name, "pngfile_logo") )  {

			data->pngfile_logo_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_logo_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					if( xmlStrcmp( xmlGetProp(cur2, "id"), "full")  )  g_free(data->pngfile_logo_filename);
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_board */
		if(! xmlStrcmp(cur->name, "pngfile_board") )  {

			data->pngfile_board_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_board_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_board_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_board_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_board_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_board_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_tokens */
		if(! xmlStrcmp(cur->name, "pngfile_tokens") )  {

			data->pngfile_token_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_token_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_token_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_token_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_token_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_token_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_cards */
		if(! xmlStrcmp(cur->name, "pngfile_cards") )  {

			data->pngfile_card_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_card_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_card_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_card_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_card_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_card_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_stairs */
		if(! xmlStrcmp(cur->name, "pngfile_stairs") )  {

			data->pngfile_stair_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_stair_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_stair_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_stair_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_stair_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_stair_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_stairs_m (mortgaged) */
		if(! xmlStrcmp(cur->name, "pngfile_stairs_m") )  {

			data->pngfile_stair_m_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_stair_m_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_stair_m_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_stair_m_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_stair_m_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_stair_m_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_commands */
		if(! xmlStrcmp(cur->name, "pngfile_commands") )  {

			data->pngfile_command_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_command_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_command_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_command_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_command_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_command_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_horiz_houses */
		if(! xmlStrcmp(cur->name, "pngfile_horiz_houses") )  {

			data->pngfile_horiz_house_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_horiz_house_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_horiz_house_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_horiz_house_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_horiz_house_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_horiz_house_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse pngfile_vert_houses */
		if(! xmlStrcmp(cur->name, "pngfile_vert_houses") )  {

			data->pngfile_vert_house_filename = g_strdup( xmlGetProp(cur, "filename") );
			data->pngfile_vert_house_need_alpha = atoi( xmlGetProp(cur, "need_alpha") );

			cur2 = cur->xmlChildrenNode;
			do {

				if(! xmlStrcmp(cur2->name, "buffer") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->pngfile_vert_house_x[ident]  = atoi( xmlGetProp(cur2, "x") );
					data->pngfile_vert_house_y[ident]  = atoi( xmlGetProp(cur2, "y") );
					data->pngfile_vert_house_width[ident]  = atoi( xmlGetProp(cur2, "width") );
					data->pngfile_vert_house_height[ident] = atoi( xmlGetProp(cur2, "height") );
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}


		/* ===== INTERFACE ===== */
		/* ===================== */

		/* parse board interface data */
		if(! xmlStrcmp(cur->name, "display_board") )  {

			data->number_estates = 0;

			data->board_width = atoi( xmlGetProp(cur, "width") );
			data->board_height = atoi( xmlGetProp(cur, "height") );

			cur2 = cur->xmlChildrenNode;

			do {

				if(! xmlStrcmp(cur2->name, "estate") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->estate[ident].xtable = atoi( xmlGetProp(cur2, "xtable") );
					data->estate[ident].ytable = atoi( xmlGetProp(cur2, "ytable") );
					data->estate[ident].x = atoi( xmlGetProp(cur2, "x") );
					data->estate[ident].y = atoi( xmlGetProp(cur2, "y") );
					data->estate[ident].buffer_board = atoi( xmlGetProp(cur2, "buffer_board") );
					data->estate[ident].xtoken = data->estate[ident].x + atoi( xmlGetProp(cur2, "xtoken") );
					data->estate[ident].ytoken = data->estate[ident].y + atoi( xmlGetProp(cur2, "ytoken") );
					data->estate[ident].xstair = data->estate[ident].x + atoi( xmlGetProp(cur2, "xstair") );
					data->estate[ident].ystair = data->estate[ident].y + atoi( xmlGetProp(cur2, "ystair") );
					data->estate[ident].xhouse = data->estate[ident].x + atoi( xmlGetProp(cur2, "xhouse") );
					data->estate[ident].yhouse = data->estate[ident].y + atoi( xmlGetProp(cur2, "yhouse") );
					if(! xmlStrcmp(xmlGetProp(cur2, "type_house"), "horizontal") )
						data->estate[ident].type_house = TYPE_HOUSE_HORIZONTAL;
					else if(! xmlStrcmp(xmlGetProp(cur2, "type_house"), "vertical") )
						data->estate[ident].type_house = TYPE_HOUSE_VERTICAL;
					else
						data->estate[ident].type_house = TYPE_HOUSE_NONE;

					data->number_estates++;
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		/* parse player token interface data */
		if(! xmlStrcmp(cur->name, "display_playertoken") )  {

			data->playerlist_token_width = atoi( xmlGetProp(cur, "width") );
			data->playerlist_token_height = atoi( xmlGetProp(cur, "height") ); 

			if( atoi(xmlGetProp(cur, "bgcolor")) < 0 )
				data->playerlist_token_bgcolor = -1;
			else  {
				sscanf(xmlGetProp(cur, "bgcolor"), "0x%6X", &color);
				data->playerlist_token_bgcolor = color;
			}
		}

		/* parse player cards interface data */
		if(! xmlStrcmp(cur->name, "display_playercard") )  {

			data->number_playerlist_card = 0;

			data->playerlist_cards_width = atoi( xmlGetProp(cur, "width") );
			data->playerlist_cards_height = atoi( xmlGetProp(cur, "height") ); 
			xoffset = atoi( xmlGetProp(cur, "xoffset") );
			yoffset = atoi( xmlGetProp(cur, "yoffset") );

			if(! g_strncasecmp(xmlGetProp(cur, "alphaunowned"), "0x", 2) )  {

				sscanf(xmlGetProp(cur, "alphaunowned"), "0x%2X", &color);
				data->playerlist_cards_alphaunowned = color;
			}
			else data->playerlist_cards_alphaunowned = atoi( xmlGetProp(cur, "alphaunowned") );

			if(! g_strncasecmp(xmlGetProp(cur, "alphaowned"), "0x", 2) )  {

				sscanf(xmlGetProp(cur, "alphaowned"), "0x%2X", &color);
				data->playerlist_cards_alphaowned = color;
			}
			else data->playerlist_cards_alphaowned = atoi( xmlGetProp(cur, "alphaowned") );

			sscanf(xmlGetProp(cur, "cardbgcolor"), "0x%6X", &color);
			data->playerlist_cards_cardbgcolor = color;

			if( atoi(xmlGetProp(cur, "bgcolor")) < 0 )
				data->playerlist_cards_bgcolor = -1;
			else  {
				sscanf(xmlGetProp(cur, "bgcolor"), "0x%6X", &color);
				data->playerlist_cards_bgcolor = color;
			}

			cur2 = cur->xmlChildrenNode;

			do {

				if(! xmlStrcmp(cur2->name, "card") )  {

					ident = atoi( xmlGetProp(cur2, "id") );

					data->playerlist_card[ident].x = atoi( xmlGetProp(cur2, "x") ) + xoffset;
					data->playerlist_card[ident].y = atoi( xmlGetProp(cur2, "y") ) + yoffset;
					data->playerlist_card[ident].z = atoi( xmlGetProp(cur2, "z") );
					data->playerlist_card[ident].buffer_card = atoi( xmlGetProp(cur2, "buffer_card") );
					data->playerlist_card[ident].estateid = atoi( xmlGetProp(cur2, "estateid") );

					data->number_playerlist_card++;
				}

				cur2 = cur2 -> next;
			} while(cur2 != NULL);
		}

		cur = cur -> next;

	} while(cur != NULL);

	xmlFreeDoc(doc);
	return TRUE;
}


/* read all intro pngs images */
gboolean intro_load_pngs()   {

	png_structp png_ptr;
	png_infop info_ptr;
	_png_imagedata png_imagedata;
	guchar *filename;
	FILE *infile;


	/* ---- load logo file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_logo_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_logo_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
	data->logo_width = png_imagedata.width;
	data->logo_height = png_imagedata.height;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	data->pngfile_logo_image = readpng_get_image(&png_imagedata);
	data->pngfile_logo_alpha = readpng_get_alpha(&png_imagedata);

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	return TRUE;
}


/* read all game pngs images */
gboolean game_load_pngs()   {

	png_structp png_ptr;
	png_infop info_ptr;
	_png_imagedata png_imagedata;
	guchar *filename;
	FILE *infile;
	guint32 i;


	/* ---- load board file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_board_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_board_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, FALSE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_board_width[i] ; i++)  {

		data->pngfile_board_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_board_x[i], data->pngfile_board_y[i], data->pngfile_board_width[i], data->pngfile_board_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	/* ---- load token file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_token_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_token_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_token_width[i] ; i++)  {

		data->pngfile_token_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_token_x[i], data->pngfile_token_y[i], data->pngfile_token_width[i], data->pngfile_token_height[i]);
		data->pngfile_token_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_token_x[i], data->pngfile_token_y[i], data->pngfile_token_width[i], data->pngfile_token_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	/* ---- load stair file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_stair_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_stair_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_stair_width[i] ; i++)  {

		data->pngfile_stair_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_stair_x[i], data->pngfile_stair_y[i], data->pngfile_stair_width[i], data->pngfile_stair_height[i]);
		data->pngfile_stair_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_stair_x[i], data->pngfile_stair_y[i], data->pngfile_stair_width[i], data->pngfile_stair_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	/* ---- load stair_m file (mortgaged) */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_stair_m_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_stair_m_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_stair_m_width[i] ; i++)  {

		data->pngfile_stair_m_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_stair_m_x[i], data->pngfile_stair_m_y[i], data->pngfile_stair_m_width[i], data->pngfile_stair_m_height[i]);
		data->pngfile_stair_m_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_stair_m_x[i], data->pngfile_stair_m_y[i], data->pngfile_stair_m_width[i], data->pngfile_stair_m_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	/* ---- load cards file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_card_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_card_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, FALSE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_card_width[i] ; i++)  {

		data->pngfile_card_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_card_x[i], data->pngfile_card_y[i], data->pngfile_card_width[i], data->pngfile_card_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	/* ---- load command file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_command_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_command_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_command_width[i] ; i++)  {

		data->pngfile_command_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_command_x[i], data->pngfile_command_y[i], data->pngfile_command_width[i], data->pngfile_command_height[i]);
		data->pngfile_command_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_command_x[i], data->pngfile_command_y[i], data->pngfile_command_width[i], data->pngfile_command_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	/* ---- load horiz_houses file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_horiz_house_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_horiz_house_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_horiz_house_width[i] ; i++)  {

		data->pngfile_horiz_house_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_horiz_house_x[i], data->pngfile_horiz_house_y[i], data->pngfile_horiz_house_width[i], data->pngfile_horiz_house_height[i]);
		data->pngfile_horiz_house_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_horiz_house_x[i], data->pngfile_horiz_house_y[i], data->pngfile_horiz_house_width[i], data->pngfile_horiz_house_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);



	/* ---- load vert_houses file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_vert_house_filename, NULL);

	if (! (infile = fopen(filename, "rb")) ) {

		fprintf(stderr, "can't open file [%s]\n", filename);
		g_free(filename);
		return FALSE;
	}
	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {

		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
		fclose(infile);
		return FALSE;
	}

	if(data->pngfile_vert_house_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;

	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {

		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
		return FALSE;
	}
	fclose(infile);

	for(i = 0 ; data->pngfile_vert_house_width[i] ; i++)  {

		data->pngfile_vert_house_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_vert_house_x[i], data->pngfile_vert_house_y[i], data->pngfile_vert_house_width[i], data->pngfile_vert_house_height[i]);
		data->pngfile_vert_house_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_vert_house_x[i], data->pngfile_vert_house_y[i], data->pngfile_vert_house_width[i], data->pngfile_vert_house_height[i]);
	}

	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);


	return TRUE;
}



void init_prototype_gtk_style()  {

	GtkWidget *label;
	GtkStyle  *rcstyle;

	/* create a dummy label so we can fetch the associated rc style */
	label = gtk_label_new ("");
	rcstyle = gtk_rc_get_style (label);
	gtk_widget_destroy (label);

	/* this is needed for some (broken?) gtk themes */
	if (rcstyle == NULL)
		rcstyle = gtk_style_new ();
	else
		gtk_style_ref (rcstyle);


	global->style_normal = gtk_style_new();

	global->style_player_normal = gtk_style_copy (rcstyle);
	global->style_player_normal->fg[0].red   = 0x0000;
	global->style_player_normal->fg[0].green = 0x0000;
	global->style_player_normal->fg[0].blue  = 0x0000;

	global->style_player_turn = gtk_style_copy (rcstyle);
	global->style_player_turn->fg[0].red   = 0xffff;
	global->style_player_turn->fg[0].green = 0x0000;
	global->style_player_turn->fg[0].blue  = 0x0000;


	gtk_style_unref (rcstyle);
}

