/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <gtk/gtk.h>

#include <libxml/parser.h>

#include "xmlparse.h"
#include "game.h"
#include "client.h"
#include "interface.h"
#include "global.h"
#include "load.h"
#include "callback.h"

#include "engine.h"

/*
	server
	client
	msg
	display
	updateplayerlist
	updategamelist
	playerupdate
	estateupdate
	cardupdate
	estategroupupdate
	tradeupdate
	auctionupdate
	gameupdate
	commandlist
	configupdate
*/

/*
	.gn
	.gl
	.gj
	.gd
	.gp
	.gs
	.gx
	.Tn
	.Tc
	.Te
	.Tm
	.Ta
	.Tr
	.r
	.E
	.t
	.D
	.p
	.eb
	.ea
	.hb
	.hs
	.em
	.ab
	.jc
	.jp
	.jr
	.T$
	.T%
	.f
	.n
	.d
	.R
*/


void xmlparse_getgamelist_plugger(guint32 connectid) {

	xmlDocPtr doc;
	xmlNodePtr cur;

	doc = xmlParseMemory(connection[connectid]->buf_received,  connection[connectid]->lenght_received);

	if(doc == NULL)  return;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		xmlFreeDoc(doc);
		return;
	}

	if( xmlStrcmp(cur->name, SERVER_XMLROOTELEMENT) )  {
		xmlFreeDoc(doc);
		return;
	}

	cur = cur->xmlChildrenNode;
	do {

		if(! xmlStrcmp(cur->name, "server") )  xmlparse_server_getlist(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "client") )  xmlparse_client(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "updategamelist") )  xmlparse_updategamelist(connectid, doc, cur);

		cur = cur -> next;

	}  while(cur != NULL);

	xmlFreeDoc(doc);
	if(connection_is_open[connectid])   {

		g_free(connection[connectid]->buf_received);
		connection[connectid]->lenght_received = 0;
	}
}




void xmlparse_game_plugger(guint32 connectid) {

	xmlDocPtr doc;
	xmlNodePtr cur;

	doc = xmlParseMemory(connection[connectid]->buf_received,  connection[connectid]->lenght_received);

	if(doc == NULL)  return;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		xmlFreeDoc(doc);
		return;
	}

	if( xmlStrcmp(cur->name, SERVER_XMLROOTELEMENT) )  {
		xmlFreeDoc(doc);
		return;
	}

	cur = cur->xmlChildrenNode;
	do {

		if(! xmlStrcmp(cur->name, "gameupdate") )  xmlparse_gameupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "server") )  xmlparse_server_game(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "client") )  xmlparse_client(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "msg") )  xmlparse_message(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "estateupdate") )  xmlparse_estateupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "playerupdate") )  xmlparse_playerupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "updateplayerlist") )  xmlparse_updateplayerlist(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "auctionupdate") )  xmlparse_auctionupdate(connectid, doc, cur);

		cur = cur -> next;

	}  while(cur != NULL);

	xmlFreeDoc(doc);
	if(connection_is_open[connectid])   {

		g_free(connection[connectid]->buf_received);
		connection[connectid]->lenght_received = 0;
	}
}




void xmlparse_server_getlist(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	gchar *sendstr;

	sscanf( xmlGetProp(cur, "version"), SERVER_VERSIONSTRING, &connection[connectid]->serverversion_major, &connection[connectid]->serverversion_minor, &connection[connectid]->serverversion_revision);

	sendstr = g_strdup(".gl\n");
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);
}




void xmlparse_server_game(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	sscanf( xmlGetProp(cur, "version"), SERVER_VERSIONSTRING, &connection[connectid]->serverversion_major, &connection[connectid]->serverversion_minor, &connection[connectid]->serverversion_revision);
}




void xmlparse_client(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	if( xmlGetProp(cur, "clientid") )
		connection[connectid]->clientid = atoi( xmlGetProp(cur, "clientid") );

	if( xmlGetProp(cur, "playerid") )
		connection[connectid]->playerid = atoi( xmlGetProp(cur, "playerid") );

	if( xmlGetProp(cur, "cookie") )
		connection[connectid]->cookie = xmlGetProp(cur, "cookie");
}




void xmlparse_gameupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur) {

	guchar *sendstr;

	if(! xmlStrcmp(xmlGetProp(cur, "status"), "config") ) {

		if(game->status != GAME_STATUS_CONFIG)  {

			game->status = GAME_STATUS_CONFIG;

			sendstr = g_strdup_printf(".n%s\n", config->nickname);
			client_send(connectid, sendstr, strlen(sendstr));
			g_free(sendstr);

			interface_create_gameconfigpage();
		}
	}

	if(! xmlStrcmp(xmlGetProp(cur, "status"), "init") )  {

		if(game->status != GAME_STATUS_INIT)  {

			game->status = GAME_STATUS_INIT;
			interface_create_gameinitpage();
			if(! game_load_pngs() ) exit(-1);
		}
	}

	if(! xmlStrcmp(xmlGetProp(cur, "status"), "run") )  {

		if(game->status != GAME_STATUS_RUN)  {

			game->status = GAME_STATUS_RUN;
			interface_create_gameboardpage();
		}
	}

	if(! xmlStrcmp(xmlGetProp(cur, "status"), "end") )  {

		if(game->status != GAME_STATUS_END)  {

			game->status = GAME_STATUS_END;
		}
	}
}




void xmlparse_message(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	gchar *type, *text, *author;

	type = g_strdup( xmlGetProp(cur, "type") );

	if(! g_strncasecmp(type, "chat", 4) )  {

		author = g_strdup( xmlGetProp(cur, "author") );
		text = g_strdup_printf("<%s> %s", author, xmlGetProp(cur, "value"));
		text_insert_chat(text, strlen(text));
		g_free(text);
		g_free(author);
	}
	else {

		g_strup( type );
		text = g_strdup_printf("%s: %s", type, xmlGetProp(cur, "value"));
		text_insert_message(text, strlen(text));
		g_free(text);
	}

	g_free(type);
}




void xmlparse_estateupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guint32 color[3], i;
	gchar *tmp;
	guint32 id, frame, pic, stair;
	gint32 t, u;

	if(! xmlGetProp(cur, "estateid") )  return;

	id = atoi( xmlGetProp(cur, "estateid") );
	if(id < 0) return;

	if( xmlGetProp(cur, "name") )
		game->estate[id].name = g_strdup( xmlGetProp(cur, "name") );

	if( xmlGetProp(cur, "color") ) {

		sscanf(xmlGetProp(cur, "color"), "#%2X%2X%2X", &color[0], &color[1], &color[2]);
		game->estate[id].color[0] = color[0];
		game->estate[id].color[1] = color[1];
		game->estate[id].color[2] = color[2];
	}

	if( xmlGetProp(cur, "bgcolor") ) {

		sscanf(xmlGetProp(cur, "bgcolor"), "#%2X%2X%2X", &color[0], &color[1], &color[2]);
		game->estate[id].bgcolor[0] = color[0];
		game->estate[id].bgcolor[1] = color[1];
		game->estate[id].bgcolor[2] = color[2];
	}

	if( xmlGetProp(cur, "owner") )  {

		t = game->estate[id].owner = atoi( xmlGetProp(cur, "owner") );
		if(t > 0)  {

			/* playerlist card */
			frame = game->player[ get_player_slot_with_playerid(t) ].playerlist_cards_frame;
			pic = game->player[ get_player_slot_with_playerid(t) ].playerlist_cards_pic[ get_playerlistcard_id_with_estate(id) ];

			pic_set_alpha(frame, pic, data->playerlist_cards_alphaowned);
		}
		else {

			/* playerlist card */
			frame = game->player[ get_player_slot_with_playerid(t) ].playerlist_cards_frame;
			pic = game->player[ get_player_slot_with_playerid(t) ].playerlist_cards_pic[ get_playerlistcard_id_with_estate(id) ];
			pic_set_alpha(frame, pic, data->playerlist_cards_alphaunowned);
		}
	}

	if( xmlGetProp(cur, "houses") )  {

		t = game->estate[id].houses = atoi( xmlGetProp(cur, "houses") );
		frame = game->board_frame;

		if(t <= 0)   {

			/* houses */
			pic = game->estate[id].house_pic;
			pic_unshow(frame, pic);
		}
		else  if(t > 0  &&  data->estate[id].type_house == TYPE_HOUSE_HORIZONTAL)  {			

			/* houses */
			pic = game->estate[id].house_alpha;
			pic_set_width(frame, pic, data->pngfile_horiz_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_horiz_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_horiz_house_alpha[t]);

			pic = game->estate[id].house_pic;
			pic_set_width(frame, pic, data->pngfile_horiz_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_horiz_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_horiz_house_image[t]);
			pic_show(frame, pic);
		}
		else  if(t > 0  &&  data->estate[id].type_house == TYPE_HOUSE_VERTICAL)  {

			/* houses */
			pic = game->estate[id].house_alpha;
			pic_set_width(frame, pic, data->pngfile_vert_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_vert_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_vert_house_alpha[t]);

			pic = game->estate[id].house_pic;
			pic_set_width(frame, pic, data->pngfile_vert_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_vert_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_vert_house_image[t]);
			pic_show(frame, pic);
		}
	}

	if( xmlGetProp(cur, "houseprice") )
		game->estate[id].houseprice = atoi( xmlGetProp(cur, "houseprice") );

	if( xmlGetProp(cur, "sellhouseprice") )
		game->estate[id].sellhouseprice = atoi( xmlGetProp(cur, "sellhouseprice") );

	if( xmlGetProp(cur, "mortgageprice") )
		game->estate[id].mortgageprice = atoi( xmlGetProp(cur, "mortgageprice") );

	if( xmlGetProp(cur, "unmortgageprice") )
		game->estate[id].unmortgageprice = atoi( xmlGetProp(cur, "unmortgageprice") );

	if( xmlGetProp(cur, "mortgaged") )  {

		u = game->estate[id].mortgaged = atoi( xmlGetProp(cur, "mortgaged") );
		t = game->estate[id].owner;
		if(t > 0  &&  !u)  {

			/* stair */
			frame = game->board_frame;
			stair = game->player[ get_player_slot_with_playerid(t) ].buffer_stair;

			pic = game->estate[id].stair_alpha;
			pic_set_width(frame, pic, data->pngfile_stair_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_alpha[stair]);

			pic = game->estate[id].stair_pic;
			pic_set_width(frame, pic, data->pngfile_stair_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_image[stair]);
			pic_show(frame, pic);
		}
		else  if(t > 0  &&  u)  {

			/* stair */
			frame = game->board_frame;
			stair = game->player[ get_player_slot_with_playerid(t) ].buffer_stair;

			pic = game->estate[id].stair_alpha;
			pic_set_width(frame, pic, data->pngfile_stair_m_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_m_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_m_alpha[stair]);

			pic = game->estate[id].stair_pic;
			pic_set_width(frame, pic, data->pngfile_stair_m_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_m_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_m_image[stair]);
			pic_show(frame, pic);
		}
		else /* if( t <= 0 ) */ {

			/* stair */
			frame = game->board_frame;
			pic_unshow(frame, game->estate[id].stair_pic);
		}
	}

	if( xmlGetProp(cur, "group") )
		game->estate[id].group = atoi( xmlGetProp(cur, "group") );

	if( xmlGetProp(cur, "can_be_owned") )
		game->estate[id].can_be_owned = atoi( xmlGetProp(cur, "can_be_owned") );

	if( xmlGetProp(cur, "can_toggle_mortgage") )
		game->estate[id].can_toggle_mortgage = atoi( xmlGetProp(cur, "can_toggle_mortgage") );

	if( xmlGetProp(cur, "can_buy_houses") )
		game->estate[id].can_buy_houses = atoi( xmlGetProp(cur, "can_buy_houses") );

	if( xmlGetProp(cur, "can_sell_houses") )
		game->estate[id].can_sell_houses = atoi( xmlGetProp(cur, "can_sell_houses") );

	if( xmlGetProp(cur, "money") )
		game->estate[id].money = atoi( xmlGetProp(cur, "money") );

	if( xmlGetProp(cur, "price") )
		game->estate[id].price = atoi( xmlGetProp(cur, "price") );

	for(i = 0 ; i <= 5 ; i++) {
   
		tmp = g_strdup_printf("rent%d", i);
		if( xmlGetProp(cur, tmp) )
			game->estate[id].rent[i] = atoi( xmlGetProp(cur, tmp) );
	}
}




void xmlparse_playerupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guint32 id, playerid;
	gchar *sendstr;

	playerid = atoi( xmlGetProp(cur, "playerid") );

	if(!playerid) return;

	id = get_player_slot_with_playerid(playerid);
	if(id < 0) return;

	if( xmlGetProp(cur, "name") )  {
		if(game->player[id].name) g_free(game->player[id].name);
		game->player[id].name = g_strndup( xmlGetProp(cur, "name"), MAX_LENGTH_NICK );

		if(game->player[id].playerlist_LabelNamePlayer)
			gtk_label_set_text(GTK_LABEL(game->player[id].playerlist_LabelNamePlayer), game->player[id].name);
	}

	if( xmlGetProp(cur, "money") )  {
		game->player[id].money = atoi( xmlGetProp(cur, "money") );

		if(game->player[id].playerlist_LabelMoneyPlayer)
			gtk_label_set_text(GTK_LABEL(game->player[id].playerlist_LabelMoneyPlayer), xmlGetProp(cur, "money"));
	}

	if( xmlGetProp(cur, "location") )  {

		game->player[id].location = atoi( xmlGetProp(cur, "location") );
		pic_set_x(game->board_frame, game->player[id].token_pic, data->estate[ game->player[id].location ].xtoken);
		pic_set_y(game->board_frame, game->player[id].token_pic, data->estate[ game->player[id].location ].ytoken);

		sendstr = g_strdup_printf(".t%d\n", game->player[id].location);
		client_send(connectid, sendstr, strlen(sendstr));
		g_free(sendstr);
	}

	if( xmlGetProp(cur, "jailed") )
		game->player[id].jailed = atoi( xmlGetProp(cur, "jailed") );

	if( xmlGetProp(cur, "directmove") )
		game->player[id].directmove = atoi( xmlGetProp(cur, "directmove") );

	if( xmlGetProp(cur, "bankrupt") )
		game->player[id].bankrupt = atoi( xmlGetProp(cur, "bankrupt") );

	if( xmlGetProp(cur, "hasturn") )  {
		game->player[id].hasturn = atoi( xmlGetProp(cur, "hasturn") );

		if(game->player[id].playerlist_LabelNamePlayer)  {

			if(game->player[id].hasturn)  {

				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementNamePlayer), "namelabel") ,  global->style_player_turn);
				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementMoneyPlayer), "moneylabel") ,  global->style_player_turn);
			}
			else  {

				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementNamePlayer), "namelabel") ,  global->style_player_normal);
				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementMoneyPlayer), "moneylabel") ,  global->style_player_normal);
			}
		}
	}

	if( xmlGetProp(cur, "can_roll") )
		game->player[id].can_roll = atoi( xmlGetProp(cur, "can_roll") );

	if( xmlGetProp(cur, "can_buyestate") )
		game->player[id].can_buyestate = atoi( xmlGetProp(cur, "can_buyestate") );
}




void xmlparse_auctionupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	GtkWidget *box, *text, *entry;
	guint32 actor, estateid, highbid, highbidder, len, status;
	guchar *ptr, *message;

	if(! xmlGetProp(cur, "auctionid") ) return;

	/* initialise auction */
	if( xmlGetProp(cur, "actor") )
		actor = atoi( xmlGetProp(cur, "actor") );

	if( xmlGetProp(cur, "estateid") )  {
		estateid = atoi( xmlGetProp(cur, "estateid") );

		interface_flush_boardcenter();
		interface_create_auctionbox();

		text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");
		entry = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_entry");

		ptr = g_strdup( xmlGetProp(cur, "auctionid") );
		gtk_object_set_data_full(GTK_OBJECT(entry), "auctionid", ptr, g_free);

		message = g_strdup_printf("%s auction estate %s", game->player[ get_player_slot_with_playerid( actor ) ].name, game->estate[ estateid ].name);
		gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );
		g_free(message);
	}

	/* bid */
	if( xmlGetProp(cur, "highbid") )
		highbid = atoi( xmlGetProp(cur, "highbid") );

	if( xmlGetProp(cur, "highbidder") )  {

		highbidder = atoi( xmlGetProp(cur, "highbidder") );

		message = g_strdup_printf("%s bid %d", game->player[ get_player_slot_with_playerid(highbidder) ].name, highbid);

		text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");

		len = gtk_text_get_length( GTK_TEXT(text) );
		gtk_text_set_point( GTK_TEXT(text), len);
		if(len > 0)  gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, "\n", 1);
		gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );

		g_free(message);
	}

	/* status */
	if( xmlGetProp(cur, "status") )  {

		status = atoi( xmlGetProp(cur, "status") );

		switch( status )  {

			case 1:
				text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");

				message = g_strdup("\n<- going once ->");
				gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );
				g_free(message);
				break;

			case 2:
				text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");

				message = g_strdup("\n<- going twice ->");
				gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );
				g_free(message);
				break;

			case 3:
				interface_flush_boardcenter();
				interface_create_message_boardcenter();
				break;
		}
	}
}




void xmlparse_updateplayerlist(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	gchar *type, *txt[10];
	xmlNodePtr cur2;
	gint8 id;
	guint32 i;
	gint32 row;

	type = g_strdup( xmlGetProp(cur, "type") );


	if(! strncmp("full", type, 4) )  {

		for(i = 0 ; i < MAX_PLAYERS ; i ++) {

			if(game->player[i].playerid > 0) {

				g_free(game->player[i].name);
				g_free(game->player[i].host);
				memset(&game->player[i], 0, sizeof(_player) );
			}
		}
		game->nb_players = 0;
	}


	if(! strncmp("add", type, 3)  ||  ! strncmp("full", type, 4) )  {

		cur2 = cur->xmlChildrenNode;
		do {
			if(! xmlGetProp(cur2, "player") ) {

				if(! game_get_valid_player_slot(&id) ) break;
				game->player[id].playerid = atoi( xmlGetProp(cur2, "playerid") );
				game->player[id].name = g_strndup( xmlGetProp(cur2, "name"), MAX_LENGTH_NICK);
				game->player[id].host = g_strdup( xmlGetProp(cur2, "host") );
				game->player[id].master = atoi( xmlGetProp(cur2, "master") );
				game->nb_players++;
			}

			cur2 = cur2 -> next;

		} while(cur2 != NULL);
	}


	if(! strncmp("del", type, 3) ) {

		cur2 = cur->xmlChildrenNode;
		do {
			if(! xmlGetProp(cur2, "player") ) {

				id = get_player_slot_with_playerid( atoi( xmlGetProp(cur2, "playerid") ) );
				if(id < 0) return;

				g_free(game->player[id].name);
				g_free(game->player[id].host);
				memset(&game->player[id], 0, sizeof(_player) );
				game->nb_players--;
			}

			cur2 = cur2 -> next;

		} while(cur2 != NULL);
	}


	if(! strncmp("edit", type, 4) ) {

		cur2 = cur->xmlChildrenNode;
		do {
			if(! xmlGetProp(cur2, "player") ) {

				id = get_player_slot_with_playerid( atoi( xmlGetProp(cur2, "playerid") ) );
				if(id < 0) return;

				if( xmlGetProp(cur2, "name") )  {
					g_free(game->player[id].name);
					game->player[id].name = g_strndup( xmlGetProp(cur2, "name"), MAX_LENGTH_NICK);
				}
				if( xmlGetProp(cur2, "host") )  {
					g_free(game->player[id].host);
					game->player[id].host = g_strdup( xmlGetProp(cur2, "host") );
				}
				if( xmlGetProp(cur2, "master") )
					game->player[id].master = atoi( xmlGetProp(cur2, "master") );
			}

			cur2 = cur2 -> next;

		} while(cur2 != NULL);
	}

	g_free(type);


	/* build player list */
	gtk_clist_freeze(GTK_CLIST(game->PlayerCList));
	gtk_clist_clear(GTK_CLIST(game->PlayerCList));

	for(i = 0 ; i < MAX_PLAYERS ; i++)  {

		if(game->player[i].playerid)  {

			txt[0] = g_strdup( game->player[i].name );
			txt[1] = g_strdup( game->player[i].host );
			row = gtk_clist_append(GTK_CLIST(game->PlayerCList), txt);
			gtk_clist_set_selectable(GTK_CLIST(game->PlayerCList), row ,FALSE);
			g_free(txt[0]);
			g_free(txt[1]);			
		}
	}

	gtk_clist_thaw(GTK_CLIST(game->PlayerCList));
}




void xmlparse_updategamelist(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	gchar *txt[10], *host, *port, **text[2];
	gint32 id, test, i;
	xmlNodePtr cur2;


	if(! xmlStrcmp(xmlGetProp(cur, "type"), "full") )  {

		gtk_clist_freeze(GTK_CLIST(global->GameCList));
		gameclist_title_showgames();

		host = g_strdup( g_strstrip(connection[connectid]->host) );
		port = g_strdup_printf("%d", connection[connectid]->port);

		/* remove all game from same host & port */
		text[0] = malloc(8192);
		text[1] = malloc(8192);
		for(i = 0 ;   ; i++) {

			test = gtk_clist_get_text(GTK_CLIST(global->GameCList), i, 0, text[0]);
			test = gtk_clist_get_text(GTK_CLIST(global->GameCList), i, 5, text[1]);
			if(!test) break;

			if(! strncmp(host, *text[0], strlen(host))  &&  ! strncmp(port, *text[1], strlen(port)))  {

				gtk_clist_remove(GTK_CLIST(global->GameCList), i);
				i--;
			}
		}
		g_free(text[0]);
		g_free(text[1]);

		cur2 = cur->xmlChildrenNode;
		do {

			if(! strncmp("game", cur2->name, 4) )  {

				if( xmlGetProp(cur2, "canbejoined")  &&  atoi( xmlGetProp(cur2, "canbejoined") ) == 0)  {
					cur2 = cur2 -> next;
					continue;
				}

				/* ONLY TEMPORARLY ! */
				if(xmlStrcmp(xmlGetProp(cur2, "gametype"), "city")  &&  xmlStrcmp(xmlGetProp(cur2, "gametype"), "french_city") )  {
					cur2 = cur2 -> next;
					continue;
				}

				txt[0] = g_strdup(host);
				txt[1] = g_strdup_printf("%d.%d.%d",  connection[connectid]->serverversion_major,  connection[connectid]->serverversion_minor,  connection[connectid]->serverversion_revision);
				id = atoi( xmlGetProp(cur2, "id") );
				if(id < 0)
					txt[2] = g_strdup_printf("Create a new %s game", xmlGetProp(cur2, "name"));
				else
					txt[2] = g_strdup_printf("Join %s game #%d", xmlGetProp(cur2, "name"), id);
				txt[3] = g_strdup(xmlGetProp(cur2, "description"));
				txt[4] = g_strdup(xmlGetProp(cur2, "players"));
				txt[5] = g_strdup(port);
				txt[6] = g_strdup(xmlGetProp(cur2, "gametype") );
				txt[7] = xmlGetProp(cur2, "id");

				gtk_clist_append(GTK_CLIST(global->GameCList), txt);

				for(i = 0 ; i < 8 ; i++)
					g_free(txt[i]);

			}
			cur2 = cur2 -> next;

		} while(cur2 != NULL);

		gtk_clist_thaw(GTK_CLIST(global->GameCList));
		g_free(host);
		g_free(port);

		server_disconnect(connectid);
	}
}




void get_metaserver_list(guint32 connectid) {

	guint32 end = 0, start = 0, lenght, lendata;
	gchar *recv, *begintext, *endtext;

	begintext = g_strdup("<monopigator>");
	endtext = g_strdup("</monopigator>");

	recv = connection[connectid]->buf_received;
	lenght = connection[connectid]->lenght_received;

	/* get end offset of monopigator xml if exist */ 
	for(end = 0 ; end < lenght - strlen(endtext) ; end++)
		if(! strncmp(endtext, recv + end, strlen(endtext) ) )  {

			end += strlen(endtext);

			/* get start offset of monopigator xml */
			for(start = 0 ; start < lenght - strlen(begintext) ; start++)
				if(! strncmp(begintext, recv + start, strlen(begintext) ) )
					break;

			break;
		}

	/* if all monopigator xml is retreived */
	if(start) {

		lendata = end - start;

		g_memmove(connection[connectid]->buf_received, connection[connectid]->buf_received + start, lendata);
		memset(connection[connectid]->buf_received + lendata, 0, lenght - lendata);

		xmlparse_metaserver(connectid);
		server_disconnect(connectid);
	}

	g_free(begintext);
	g_free(endtext);
}




void xmlparse_metaserver(guint32 connectid) {

	gchar *txt[10];
	xmlDocPtr doc;
	xmlNodePtr cur;
	guint16 connectid2;
	guint32 i;

	doc = xmlParseMemory(connection[connectid]->buf_received,  connection[connectid]->lenght_received);

	if(doc == NULL)  return;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {

		xmlFreeDoc(doc);
		return;
	}

	if( xmlStrcmp(cur->name, METASERVER_XMLROOTELEMENT) )  {

		xmlFreeDoc(doc);
		return;
	}

	gtk_clist_freeze(GTK_CLIST(global->GameCList));

	switch(connection[connectid]->type)  {

		case CONNECT_TYPE_META_GETLIST:
			gameclist_title_showservers();
			gtk_clist_clear(GTK_CLIST(global->GameCList));

			cur = cur->xmlChildrenNode;
			do {
				if(! xmlStrcmp(cur->name, "server") )  {

					txt[0] = g_strdup(xmlGetProp(cur, "host"));
					txt[1] = g_strdup(xmlGetProp(cur, "port"));
					txt[2] = g_strdup(xmlGetProp(cur, "version"));
					txt[3] = g_strdup(xmlGetProp(cur, "users"));
					txt[4] = g_strdup("");
					gtk_clist_append(GTK_CLIST(global->GameCList), txt);

					for(i = 0 ; i < 5 ; i++)
						g_free(txt[i]);
				}

				cur = cur -> next;

			}  while(cur != NULL);
			break;


		case CONNECT_TYPE_META_GETGAME:
			gameclist_title_showgames();

			cur = cur->xmlChildrenNode;
			do {
				if(! xmlStrcmp(cur->name, "server") )
					if( server_connect(&connectid2, xmlGetProp(cur, "host"), atoi( xmlGetProp(cur, "port") ) ) )
						connection[connectid2]->type = CONNECT_TYPE_MONOPD_GETGAME;

				cur = cur -> next;

			} while(cur != NULL);
			break;


		default: fprintf(stderr, "xmlparse: unknow type connection !\n");
			break;
	}

	gtk_clist_thaw(GTK_CLIST(global->GameCList));

	xmlFreeDoc(doc);
}



